using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Runtime.InteropServices;
using System.Windows.Interop;
using System.Windows.Threading;

namespace NLarge
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// Rob Burke
    /// robburke@microsoft.com
    /// Released under MS-PL (Microsoft permissive license):  http://www.microsoft.com/resources/sharedsource/licensingbasics/permissivelicense.mspx 
    /// Copyright (c) Microsoft Corporation.  All rights reserved.
    /// </summary>
    public partial class Window1 : System.Windows.Window
    {

        public Window1()
        {
            InitializeComponent();
        }
        protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
        {
            AboutNLarge.KillAboutDialog();
            IconHolder.NI.Visible = false;
            base.OnClosing(e);
        }

        private NotifyIconHolder IconHolder;

        private void HandleFormLoad(object sender, RoutedEventArgs e)
        {
            System.Drawing.Point tempPnt = new System.Drawing.Point(PrimaryScreenWidth/2, PrimaryScreenHeight/2);
            SpotGrabInstance = new SpotGrab(tempPnt, PrimaryScreenWidth, PrimaryScreenHeight);


            UserActivityHookInstance = new UserActivityHook();
            UserActivityHookInstance.OnMouseActivity += new System.Windows.Forms.MouseEventHandler(MouseMoveHandler);
            UserActivityHookInstance.KeyDown += new System.Windows.Forms.KeyEventHandler(GlobalKeyDown);
            UserActivityHookInstance.KeyUp += new System.Windows.Forms.KeyEventHandler(GlobalKeyUp);

            intervalTimer.Interval = TimeSpan.FromMilliseconds(50);
            intervalTimer.Tick += new EventHandler(HandleCustomAnimationTimer);

            this.Focus();
            SketchInkCanvas.Focus();

            ZoomActive = false;
            IconHolder = new NotifyIconHolder(this);

            if (!HasRunBefore)
            {
                AboutNLarge.PresentAboutDialog();
                IndicateHasRunBefore();
            }
        }



        private DispatcherTimer intervalTimer = new DispatcherTimer();
        UserActivityHook UserActivityHookInstance;
        SpotGrab SpotGrabInstance;


        private void CaptureScreenImage()
        {
            SpotGrabInstance.SpotSize = new System.Drawing.Size(PrimaryScreenWidth, PrimaryScreenHeight);
            SpotGrabInstance.CursorPosition= new System.Drawing.Point(PrimaryScreenWidth/2, PrimaryScreenHeight/2);
            SpotGrabInstance.Capture();
            ZoomImageBrush.Source = SpotGrabInstance.CurrentSpotBmpSrc;
        }

        private bool ZoomActiveRep;
        public bool ZoomActive
        {
            get { return ZoomActiveRep; }
            set 
            { 
                ZoomActiveRep = value;
                if (value)
                {
                    CaptureScreenImage();
                    intervalTimer.Start();
                    IsTransformFrozen = false;
                    CurrentScale = 1;
                    DesiredScale = DesiredScaleCurrent;
                    CurrentTransformX = CurrentMousePosition.X;
                    CurrentTransformY = CurrentMousePosition.Y;
                    DesiredTransformX = CurrentMousePosition.X;
                    DesiredTransformY = CurrentMousePosition.Y;
                    UpdateTransform();
                    SketchInkCanvas.Width = PrimaryScreenWidth;
                    SketchInkCanvas.Height = PrimaryScreenHeight;

//                    this.Visibility = Visibility.Collapsed;
//                    this.WindowState = WindowState.Minimized;
                    this.WindowState = WindowState.Maximized;
                    this.Visibility = Visibility.Visible;
                    this.Focus();
                    SketchInkCanvas.Focus();
                }
                else
                {
                    intervalTimer.Stop();
                    this.Visibility = Visibility.Hidden;
                    CurrentScale = 1;
                    DesiredScale = 1;
                    UpdateTransform();
                }
            }
        }

        bool IsTransformFrozen = false;

        bool IsVanishingOnTimer = false;
        DateTime TimeVanishingStarted = DateTime.Now;


        private void HandleCustomAnimationTimer(object sender, EventArgs e)
        {
            double zoomRate = ZoomSpeed;
            double zoomRateOnVanish = zoomRate * 0.9375;
            double panRate = 0.6;
            double currentZoomRate = IsVanishingOnTimer ? zoomRateOnVanish : zoomRate;

            if (!IsTransformFrozen)
            {
                CurrentScale = CurrentScale * currentZoomRate + DesiredScale * (1 - currentZoomRate);
                CurrentTransformX = CurrentTransformX * panRate + DesiredTransformX * (1 - panRate);
                CurrentTransformY = CurrentTransformY * panRate + DesiredTransformY * (1 - panRate);
            }
                
            UpdateTransform();

            if (IsVanishingOnTimer && (CurrentScale < 1.01 || DateTime.Now > TimeVanishingStarted.AddMilliseconds(2000)))
            {
                ZoomActive = false;
            }
        }

        private bool AltKeyPressed = false;

        public void GlobalKeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
        {
            if (e.KeyData == System.Windows.Forms.Keys.LMenu)
            {
                AltKeyPressed = true;
            }
            if (AltKeyPressed)
            {
                if (e.KeyData == System.Windows.Forms.Keys.D1 || e.KeyData == System.Windows.Forms.Keys.D2)
                {
                    if (!ZoomActive)
                    {
                        bool shouldZoomIn = e.KeyData != System.Windows.Forms.Keys.D2;
                        BeginNewZoomIn(shouldZoomIn);
                        e.Handled = true;

                    }
                }
            }

        }

        internal void BeginNewZoomIn(bool shouldZoomIn)
        {
            IsVanishingOnTimer = false;
            DesiredScaleCurrent = shouldZoomIn ? ZoomDepth : 1;
            ZoomActive = true;
        }

        public void GlobalKeyUp(object sender, System.Windows.Forms.KeyEventArgs e)
        {
            if (e.KeyData == System.Windows.Forms.Keys.LMenu)
            {
                AltKeyPressed = false;
            }
        }

        private void StartVanishingOnTimer()
        {
            SketchInkCanvas.Strokes.Clear();
            IsTransformFrozen = false;
            IsVanishingOnTimer = true;
            TimeVanishingStarted = DateTime.Now;
            DesiredScale = 1;
        }

        public static int PrimaryScreenWidth
        {
            get { return System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width; }
        }
        public static int PrimaryScreenHeight
        {
            get { return System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height; }
        }

        public static double PrimaryScreenAspectRatio
        {
            get { return (double)PrimaryScreenWidth / (double)PrimaryScreenHeight; }
        }

        private System.Drawing.Point m_CurrentMousePosition;

        public System.Drawing.Point CurrentMousePosition
        {
            get { return m_CurrentMousePosition; }
            set { 
                m_CurrentMousePosition = value;

                if (!IsTransformFrozen && !IsVanishingOnTimer)
                {
                    GetDesiredTransformXYFromCurrentMousePosition();
                }

                ConstrainDesiredTransformXY();

                UpdateTransform();
            }
        }

        private void GetDesiredTransformXYFromCurrentMousePosition()
        {
            DesiredTransformX = CurrentMousePosition.X;
            DesiredTransformY = CurrentMousePosition.Y;
        }

        private void ConstrainDesiredTransformXY()
        {
            double maxTransformXTarget, maxTransformYTarget;
            GetMaxTransformsForScale(out maxTransformXTarget, out maxTransformYTarget, DesiredScale);
            double transformX = (PrimaryScreenWidth / 2 - DesiredTransformX);
            double transformY = (PrimaryScreenHeight / 2 - DesiredTransformY);

            //this.DebugLabel.Content = "MaxXTarget " + maxTransformXTarget + " DesiredtransformX " + DesiredTransformX;

            // There was a bug in here.
            //if (transformX > maxTransformXTarget) DesiredTransformX = PrimaryScreenWidth / 2 - maxTransformXTarget;
            //if (transformX < -maxTransformXTarget) DesiredTransformX = PrimaryScreenWidth / 2 + maxTransformXTarget;
            //if (transformY > maxTransformYTarget) DesiredTransformY = PrimaryScreenHeight / 2 - maxTransformYTarget;
            //if (transformY < -maxTransformYTarget) DesiredTransformY = PrimaryScreenHeight / 2 + maxTransformYTarget;
            //if (transformX > maxTransformXTarget) DesiredTransformX = maxTransformXTarget;
            //if (transformX < -maxTransformXTarget) DesiredTransformX = -maxTransformXTarget;
            //if (transformY > maxTransformYTarget) DesiredTransformY = maxTransformYTarget;
            //if (transformY < -maxTransformYTarget) DesiredTransformY = -maxTransformYTarget;

        }

        private void UpdateTransform()
        {
            TransformGroup tg = new TransformGroup();

            double transformX = (PrimaryScreenWidth / 2 - CurrentTransformX);
            double transformY = (PrimaryScreenHeight / 2 - CurrentTransformY);

            double maxTransformXCurrent, maxTransformYCurrent;
//            double maxTransformXTarget, maxTransformYTarget;
            GetMaxTransformsForScale(out maxTransformXCurrent, out maxTransformYCurrent, CurrentScale);
//            GetMaxTransformsForScale(out maxTransformXTarget, out maxTransformYTarget, DesiredScale);
            //double maxTransformX = maxTransformXTarget > 0
            //    ? Math.Min(maxTransformXCurrent, maxTransformXTarget)
            //    : Math.Max(maxTransformXCurrent, maxTransformXTarget);
            //double maxTransformY = maxTransformYTarget > 0
            //    ? Math.Min(maxTransformYCurrent, maxTransformYTarget)
            //    : Math.Max(maxTransformYCurrent, maxTransformYTarget);
            double maxTransformX = maxTransformXCurrent;
            double maxTransformY = maxTransformYCurrent;

            if (transformX > maxTransformX) transformX = maxTransformX;
            if (transformX < -maxTransformX) transformX = -maxTransformX;
            if (transformY > maxTransformY) transformY = maxTransformY;
            if (transformY < -maxTransformY) transformY = -maxTransformY;

            TranslateTransform tt = new TranslateTransform(
                transformX,
                transformY);

            ScaleTransform st = new ScaleTransform(CurrentScale, CurrentScale);

            tg.Children.Add(tt);
            tg.Children.Add(st);
            ZoomImageBrush.RenderTransform = tg;
            ZoomImageBrush.RenderTransformOrigin = new Point(0.5, 0.5);
            
        }

        private void GetMaxTransformsForScale(out double maxTransformX, out double maxTransformY, double scale)
        {
            maxTransformX = PrimaryScreenWidth * 0.5 * (CurrentScale - 1) / scale;
            maxTransformY = PrimaryScreenHeight * 0.5 * (CurrentScale - 1) / scale;
        }

        double CurrentScale = 1;

        private double m_DesiredScale = 1;
        public double DesiredScale
        {
            get { return m_DesiredScale; }
            set
            {
                m_DesiredScale = value;
                ConstrainDesiredTransformXY();
            }
        }
        double DesiredScaleCurrent = 2; 
        double CurrentTransformX = 0;
        double CurrentTransformY = 0;
        double DesiredTransformX = 0;
        double DesiredTransformY = 0;

        public void MouseMoveHandler(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            CurrentMousePosition = new System.Drawing.Point(e.X, e.Y);
            UpdateTransform();
        }

        public void HandleMouseDown(object sender, RoutedEventArgs e)
        {
            IsTransformFrozen = true;

            MouseButtonEventArgs me = e as MouseButtonEventArgs;
            if (me.RightButton == MouseButtonState.Pressed)
            {
                StartVanishingOnTimer();
            }
        }


        public void HandleMouseUp(object sender, RoutedEventArgs e)
        {
        }

        public void HandleMouseMove(object sender, RoutedEventArgs e)
        {
        }

        public void HandleMouseWheel(object sender, RoutedEventArgs e)
        {
            MouseWheelEventArgs me = e as MouseWheelEventArgs;
            if (me.Delta > 0) ZoomIn();
            else ZoomOut();
        }

        public void HandleKeyDown(object sender, RoutedEventArgs e)
        {
            KeyEventArgs ke = e as KeyEventArgs;
            if (ke != null)
            {
                if (ke.Key == Key.Up || ke.Key == Key.Right || ke.Key == Key.OemPlus)
                {
                    ZoomIn();
                }
                if (ke.Key == Key.Down || ke.Key == Key.Left || ke.Key == Key.OemMinus)
                {
                    ZoomOut();
                }
                if (ke.Key == Key.Escape)
                {
                    StartVanishingOnTimer();
                }
                if (ke.Key == Key.E)
                {
                    SketchInkCanvas.Strokes.Clear();
                }
                if (ke.Key == Key.C)
                {
                    SketchInkCanvas.Strokes.Clear();
                    if (IsTransformFrozen) IsTransformFrozen = false;
                }
            }
        }

        public void HandleLostFocus(object sender, RoutedEventArgs e)
        {
            if (ZoomActive) StartVanishingOnTimer();
        }

        public void ZoomOut()
        {
            DesiredScale *= 0.75;
            if (DesiredScale < 1) DesiredScale = 1;
        }

        public void ZoomIn()
        {
            DesiredScale /= 0.75;
            if (DesiredScale > 8) DesiredScale = 8;
        }

        public static double ZoomSpeed
        {
            get
            {
                if (ZoomSpeedRep < 0)
                {
                    string s = RegistryHelp.ReadEntry("ZoomSpeed");
                    double zoomFromRegistry = 0.65;
                    bool isInRegistry = Double.TryParse(s, out zoomFromRegistry);
                    if (!isInRegistry)
                    {
                        zoomFromRegistry = 0.65;
                        ZoomSpeed = zoomFromRegistry;
                    }
                    ZoomSpeedRep = zoomFromRegistry;
                }
                return ZoomSpeedRep;
            }
            set
            {
                RegistryHelp.WriteEntry("ZoomSpeed", value.ToString());
                ZoomSpeedRep = value;
            }
        }
        private static double ZoomSpeedRep = -1;

        public static double ZoomDepth
        {
            get
            {
                if (ZoomDepthRep < 0)
                {
                    string s = RegistryHelp.ReadEntry("ZoomDepth");
                    double zoomFromRegistry = 2.0;
                    bool isInRegistry = Double.TryParse(s, out zoomFromRegistry);
                    if (!isInRegistry)
                    {
                        zoomFromRegistry = 2.0;
                        ZoomDepth = zoomFromRegistry;
                    }
                    ZoomDepthRep = zoomFromRegistry;
                }
                return ZoomDepthRep;
            }
            set
            {
                RegistryHelp.WriteEntry("ZoomDepth", value.ToString());
                ZoomDepthRep = value;
            }
        }
        private static double ZoomDepthRep = -1;



        public bool HasRunBefore
        {
            get
            {
                string s = RegistryHelp.ReadEntry("HasRunBefore");
                if (s == null) return false;
                return true;
            }
        }
        public void IndicateHasRunBefore()
        {
            RegistryHelp.WriteEntry("HasRunBefore", "true");
        }

    }
}